using System;
using System.Xml;

namespace com.ibm.gre
{
	/// <summary>
	/// The OptionalFields class represents
	/// bit mask values specifying
	/// fields that can optionaly be added and
	/// displayed for a Life Cycle Query.
	/// These elements can be found as members of the "GetEligibleFilePlanComponents"
	/// element part of the input to the GET_ELIGIBLE_FILE_PLAN_COMPONENTS
	/// BatchController task.
	/// </summary>
	public class OptionalFields
	{
		/// <summary>
		/// Indicates No Optional Fields
		/// </summary>
		public static int NO_FIELDS = 0;
		/// <summary>
		/// Indicates the Reference Date To should be displayed for input. 
		/// </summary>
		public static int REFERENCE_DATE_TO = 1;
		/// <summary>
		/// Indicates the Reference Date From should be displayed for input.
		/// </summary>
		public static int REFERENCE_DATE_FROM = 2;
		/// <summary>
		/// Indicates the Disposal Authority field should be displayed for input.
		/// </summary>
		public static int DISPOSAL_AUTHORITY = 4;
		/// <summary>
		/// Indicates the Eligible for Cutt Off checkbox should be displayed for input.
		/// </summary>
		public static int ELIGIBLE_FOR_CUT_OFF = 8;
		/// <summary>
		/// Indicates the Eligible for close checkbox should be displayed for input.
		/// </summary>
		public static int ELIGIBLE_FOR_CLOSE = 16;
		/// <summary>
		/// Indicates the Eligible for Transition checkbox should be displayed for input.
		/// </summary>
		public static int ELIGIBLE_FOR_TRANSITION = 32;
		/// <summary>
		/// Indicates the Phase list should be displayed for input.
		/// </summary>
		public static int PHASE_ID = 64;
		public static int LIFE_CYCLE_QUERY_TYPE = 128;

		/// <summary>
		/// Constructor
		/// </summary>
		/// <param name="mask">The bit mask containing the optional fields to display.</param>
		public OptionalFields(int mask)
		{
			m_mask = mask;
		}

		/// <summary>
		/// Returns true if the Reference Date From field should be displayed for edit.
		/// </summary>
		/// <returns></returns>
		public bool editReferenceDateFrom()
		{
			return ((m_mask & REFERENCE_DATE_FROM) == REFERENCE_DATE_FROM);
		}
		/// <summary>
		/// Returns true if the Reference Date To field should be displayed for edit.
		/// </summary>
		/// <returns></returns>
		public bool editReferenceDateTo()
		{
			return ((m_mask & REFERENCE_DATE_TO) == REFERENCE_DATE_TO);	
		}
		/// <summary>
		/// Returns true if the Disposal Authority field should be displayed for edit.
		/// </summary>
		/// <returns></returns>
		public bool editDisposalAuthority()
		{
			return ((m_mask & DISPOSAL_AUTHORITY) == DISPOSAL_AUTHORITY);	
		}
		/// <summary>
		/// Returns true if the Eligible For Cutoff field should be displayed for edit.
		/// </summary>
		/// <returns></returns>
		public bool editEligibleForCutOff()
		{
			return ((m_mask & ELIGIBLE_FOR_CUT_OFF) == ELIGIBLE_FOR_CUT_OFF);	
		}
		/// <summary>
		/// Returns true if the Eligible for Close field should be displayed for edit.
		/// </summary>
		/// <returns></returns>
		public bool editEligibleForClose()
		{
			return ((m_mask & ELIGIBLE_FOR_CLOSE) == ELIGIBLE_FOR_CLOSE);	
		}
		/// <summary>
		/// Returns true if the Eligible for Transition field should be displayed for edit.
		/// </summary>
		/// <returns></returns>
		public bool editEligibleForTransition()
		{
			return ((m_mask & ELIGIBLE_FOR_TRANSITION) == ELIGIBLE_FOR_TRANSITION);	
		}
		/// <summary>
		/// Returns true if the Phase ID field should be displayed for edit.
		/// </summary>
		/// <returns></returns>
		public bool editPhaseId()
		{
			return ((m_mask & PHASE_ID) == PHASE_ID);	
		}
		/// <summary>
		/// Returns true if the Life Cycle Query type field should be displayed for edit.
		/// </summary>
		/// <returns></returns>
		public bool editLifeCycleQueryType()
		{
			return ((m_mask & LIFE_CYCLE_QUERY_TYPE) == LIFE_CYCLE_QUERY_TYPE);	
		}

		private int m_mask;

	}

	/// <summary>
	/// The IRMBatchQuery class encapsulates the XML used to
	/// execute a Batch Life Cycle Query.
	/// There are methods for begininning the Asynchronous task
	/// and subsequent methods for determining the status and getting
	/// return values.
	/// </summary>
	/// <remarks>
	/// See the documentation for the BatchControllerEJB interface in
	/// the com.ibm.gre.engine.ejb.facade package conatined in
	/// the IBM DB2 Records Manager API Reference. 
	/// </remarks>
	public class IRMBatchQuery :IRMQuery
	{
		/// <summary>
		/// Constructor
		/// </summary>
		/// <param name="queryXML">The XML string containing the values making up the Query.
		/// This string can be a saved query value returned by the QueryControllerEJB.getQueryById method.
		/// </param>
		/// <param name="session">A ClientSession object containing references to the Records Manager API objects.</param>
		/// <param name="strScope">The full path string indicating the scope of the query.</param>
		/// <param name="addDisplayInfo">Set to true if you wish to use this object to generate display information.</param>
		/// <param name="optionalFieldsMask">The bit mask values indicating the optional fields to use for the Life Cycle Query.</param>
		public IRMBatchQuery(String queryXML, ClientSession session,string strScope , bool addDisplayInfo, int optionalFieldsMask):
			base(queryXML, session, strScope , addDisplayInfo)
		{
			m_optionalFields = new OptionalFields(optionalFieldsMask);
			m_nStatus = (int)IRMConstants.BatchResultStatusTypes.RECEIVED;
		}

		/// <summary>
		/// Call the Initialize method immediately after the class has been constructed
		/// to generate the internal representation of the query.
		/// </summary>
		new	public void Initialize()
		{
			//Add optional Life Cycle Fields if this is a Life Cycle Query
			if((IRMConstants.QueryTypes)m_nQueryType == IRMConstants.QueryTypes.LIFE_CYCLE)
			{
				//TODO the captions should be more carefully constructed
				XmlElement lm = (XmlElement)m_rootReportTemplate.SelectSingleNode("GetEligibleFilePlanComponents");
				if(m_optionalFields.editReferenceDateFrom())
				{
					AddOptionalField(lm,(int)IRMConstants.AttributeDataTypes.DATE,"ReferenceDateFrom","ReferenceDate is From",(int)IRMConstants.ProfileControlType.SINGLELINE);
				}
				if(m_optionalFields.editReferenceDateTo())
				{
					string captn = "To";
					if(!m_optionalFields.editReferenceDateFrom())
					{
						captn = "ReferenceDate Is";
					}
					AddOptionalField(lm,(int)IRMConstants.AttributeDataTypes.DATE,"ReferenceDateTo",captn,(int)IRMConstants.ProfileControlType.SINGLELINE);
				}
				if(m_optionalFields.editDisposalAuthority())
				{
					AddOptionalField(lm,(int)IRMConstants.AttributeDataTypes.BOOLEAN,"IncludeDisposalAuthority","IncludeDisposalAuthority",(int)IRMConstants.ProfileControlType.SINGLELINE);
				}
				if(m_optionalFields.editEligibleForCutOff())
				{
					AddOptionalField(lm,(int)IRMConstants.AttributeDataTypes.BOOLEAN,"EligibleForCutoff","EligibleForCutoff",(int)IRMConstants.ProfileControlType.SINGLELINE);
				}
				if(m_optionalFields.editEligibleForClose())
				{
					AddOptionalField(lm,(int)IRMConstants.AttributeDataTypes.BOOLEAN,"EligibleForClose","EligibleForClose",(int)IRMConstants.ProfileControlType.SINGLELINE);
				}
				if(m_optionalFields.editEligibleForTransition())
				{
					AddOptionalField(lm,(int)IRMConstants.AttributeDataTypes.BOOLEAN,"EligibleForTransition","EligibleForTransition",(int)IRMConstants.ProfileControlType.SINGLELINE);
				}
				if(m_optionalFields.editPhaseId())
				{
					AddOptionalField(lm,(int)IRMConstants.AttributeDataTypes.INTEGER,"PhaseId","PhaseId",(int)IRMConstants.ProfileControlType.SINGLELINE);
				}
				if(m_optionalFields.editLifeCycleQueryType())
				{
					AddOptionalField(lm,(int)IRMConstants.AttributeDataTypes.INTEGER,"LifeCycleQueryType","LifeCycleQueryType",(int)IRMConstants.ProfileControlType.SINGLELINE);
				}	
			}
			
			//Then initialize the base so that the new fields are always first
			base.Initialize();
		}

		/// <summary>
		/// Used internally in the Initialize method to add optional fields
		/// </summary>
		/// <param name="lm">The XML element for the field</param>
		/// <param name="dataType">data type</param>
		/// <param name="key">The name of the element used in the GetEligibleFilePlanComponents element of the Query XML</param>
		/// <param name="caption">Caption</param>
		/// <param name="nControlType">Typ of control to use.</param>
		private void AddOptionalField(XmlElement lm, int dataType, string key, string caption, int nControlType)
		{
			XmlElement fieldElement = (XmlElement)lm.SelectSingleNode(key);
			Line ln = new Line(fieldElement,dataType,key,caption,0,(int)IRMConstants.LineType.NONE,nControlType,m_tab);
			m_fieldvalues.Add(key,ln);
			if(m_addDisplayInfo)
			{
				m_DisplayItems.Add(ln);
			}
		}

		/// <summary>
		/// Overrides the IRMQuery runQuery method.
		/// If the Asynchronous task has not begun this method will
		/// call the BatchController beginTask method to start the
		/// Life Cycle Query.
		/// </summary>
		/// <param name="nBlockStart">The index value return a block of values from. 0 indicates the first value.</param>
		/// <param name="nBlockSize">The number of values to return. 0 indicates return all values.</param>
		/// <returns>An integer value indicating the status of the task.
		/// Positive values indicate percent complete.
		/// Negative values refer to BatchResultStatusTypes.
		/// </returns>
		new public int runQuery(int nBlockStart, int nBlockSize)
		{
			BlockStart = nBlockStart;
			BlockSize = nBlockSize;
			if(m_strTaskKey == null)
			{
				m_strTaskName = "Get Elegible " + base.FlPlnCmpntDefName + "s " + System.DateTime.Now;
				m_strTaskKey = m_session.getBatchController().beginTask
					(
						(int)IRMConstants.AsynchronousTaskType.GET_ELIGIBLE_FILE_PLAN_COMPONENTS,
						base.ModifiedQuery,
						m_strTaskName,
						""
					);
			}
			reLoad();
			return m_nStatus;
		}

		/// <summary>
		/// The ID of the current task.
		/// </summary>
		public string TaskId
		{
			get
			{
				return m_strTaskKey;
			}
		}

		/// <summary>
		/// The name of the current task
		/// </summary>
		public string TaskName
		{
			get
			{
				return m_strTaskName;
			}
		}

		/// <summary>
		/// Calls the BatchController getStatusListByTaskId method and extracts relevant
		/// values from the returned xml string.
		/// </summary>
		private void reLoad()
		{
			string xml = m_session.getBatchController().getStatusListByTaskId(m_strTaskKey);
			XmlDocument doc = new XmlDocument();
			doc.LoadXml(xml);
			XmlElement root = doc.DocumentElement;
			XmlNodeList list = root.SelectNodes("Item");
			
			if(list.Count > 0)
			{
				XmlElement item = (XmlElement)list.Item(list.Count - 1);

				m_strStartTime = item.Attributes.GetNamedItem("StartTime").Value;
				m_nStatus = System.Convert.ToInt32(item.Attributes.GetNamedItem("Status").Value);
				m_strStatusTime = item.Attributes.GetNamedItem("StatusTime").Value;
				m_strTaskID = item.Attributes.GetNamedItem("TaskID").Value;
				m_strTaskName = item.Attributes.GetNamedItem("TaskName").Value;
				m_nType = System.Convert.ToInt32(item.Attributes.GetNamedItem("Type").Value);
				m_nUserID = System.Convert.ToInt64(item.Attributes.GetNamedItem("UserID").Value);

				m_totalResults = 0;
				if(m_nStatus == 100)
				{
					m_totalResults = System.Convert.ToInt32(item.Attributes.GetNamedItem("ResultCount").Value);
					m_strResultList = m_session.getBatchController().getTaskResultsByID(m_strTaskID,(int)IRMConstants.LogTypes.RESULT_LOG,base.BlockStart,base.BlockSize);
				}
			}
		}

		private OptionalFields m_optionalFields;
		private string m_strTaskKey = null;

		private string m_strStartTime;
		private int m_nStatus;
		private string m_strStatusTime;
		private string m_strTaskID;
		private string m_strTaskName;
		private int m_nType;
		private long m_nUserID;

	}
}
